@@ -54,8 +54,8 @@ from qtpy.QtWidgets import (
 from guidata.config import _
 from guidata.configtools import get_icon, get_image_file_path, get_image_layout
 from guidata.dataset.conv import restore_dataset, update_dataset
-from guidata.dataset.datatypes import DataItemVariable
-from guidata.qthelpers import get_std_icon
+from guidata.dataset.datatypes import ComputedProp, DataItemVariable
+from guidata.qthelpers import get_std_icon, is_dark_theme
 from guidata.utils.misc import convert_date_format
 from guidata.widgets.arrayeditor import ArrayEditor
 
@@ -68,6 +68,35 @@ from guidata.widgets.arrayeditor import ArrayEditor
 
 # XXX: consider providing an interface here...
 
+
+def _get_readonly_stylesheet() -> str:
+    """Get CSS stylesheet for read-only text widgets.
+
+    Returns CSS that automatically applies styling based on readOnly property.
+
+    Returns:
+        CSS stylesheet string
+    """
+    if is_dark_theme():
+        # Dark theme colors
+        bg_color = "#505050"  # Darker gray background
+        text_color = "#cccccc"  # Light gray text
+    else:
+        # Light theme colors
+        bg_color = "#f8f8f8"  # Light gray background
+        text_color = "#333333"  # Darker gray text
+    return f"""
+        QLineEdit[readOnly="true"] {{
+            background-color: {bg_color};
+            color: {text_color};
+        }}
+        QTextEdit[readOnly="true"] {{
+            background-color: {bg_color};
+            color: {text_color};
+        }}
+    """
+
+
 if TYPE_CHECKING:
     from guidata.dataset.qtwidgets import DataSetEditLayout
 
@@ -152,7 +181,9 @@ class AbstractDataSetWidget:
         Returns:
             True if associated dataset is readonly
         """
-        return self.item.instance.is_readonly()
+        return self.item.instance.is_readonly() or self.item.get_prop_value(
+            "display", "readonly", False
+        )
 
     def check(self) -> bool:
         """Item validator
@@ -164,6 +195,11 @@ class AbstractDataSetWidget:
 
     def set(self) -> None:
         """Update data item value from widget contents"""
+        # Skip setting computed items as they are read-only
+        computed_prop = self.item.get_prop("data", "computed", None)
+        if isinstance(computed_prop, ComputedProp):
+            return  # Don't try to set computed items
+
         # XXX: consider using item.set instead of item.set_from_string...
         self.item.set_from_string(self.value())
 
@@ -194,6 +230,31 @@ class AbstractDataSetWidget:
         if not self.build_mode:
             self.parent_layout.widget_value_changed()
 
+    def retrieve_top_level_layout(self) -> DataSetEditLayout:
+        """Retrieve the top-level layout associated with this widget.
+
+        If the current widget is part of a group, this method returns the parent layout
+        of the group widget. Otherwise, it returns the immediate parent layout.
+
+        Returns:
+            DataSetEditLayout: The top-level layout for this widget.
+        """
+        top_level_layout = self.parent_layout
+        if self.parent_layout.group_widget is not None:
+            # If we are in a group, we need to iterate over widgets of this group but
+            # also over widgets of the other groups
+            top_level_layout = self.parent_layout.group_widget.parent_layout
+        return top_level_layout
+
+    def contains_computed_items(self) -> bool:
+        """Check if there are any computed items in the layout."""
+        widgets = self.retrieve_top_level_layout().get_terminal_widgets()
+        for widget in widgets:
+            computed_prop = widget.item.get_prop("data", "computed", None)
+            if isinstance(computed_prop, ComputedProp):
+                return True
+        return False
+
 
 class GroupWidget(AbstractDataSetWidget):
     """GroupItem widget
@@ -220,6 +281,7 @@ class GroupWidget(AbstractDataSetWidget):
             self.layout,
             item.item.group,
             change_callback=self.notify_value_change,
+            group_widget=self,
         )
         self.group.setLayout(self.layout)
 
@@ -353,13 +415,15 @@ class TabGroupWidget(AbstractDataSetWidget):
 def _display_callback(widget: AbstractDataSetWidget, value):
     """Handling of display callback"""
     cb = widget.item.get_prop_value("display", "callback", None)
-    if cb is not None:
+    if widget.contains_computed_items() or cb is not None:
+        top_level_layout = widget.retrieve_top_level_layout()
         if widget.build_mode:
             widget.set()
         else:
-            widget.parent_layout.update_dataitems()
-        cb(widget.item.instance, widget.item.item, value)
-        widget.parent_layout.update_widgets(except_this_one=widget)
+            top_level_layout.update_dataitems()
+        if cb is not None:
+            cb(widget.item.instance, widget.item.item, value)
+        top_level_layout.update_widgets(except_this_one=widget)
 
 
 class LineEditWidget(AbstractDataSetWidget):
@@ -376,6 +440,7 @@ class LineEditWidget(AbstractDataSetWidget):
         password = self.item.get_prop_value("display", "password", False)
         if password:
             self.edit.setEchoMode(QLineEdit.Password)
+        self.edit.setStyleSheet(_get_readonly_stylesheet())
         self.edit.textChanged.connect(self.line_edit_changed)  # type:ignore
 
     def get(self) -> None:
@@ -389,7 +454,13 @@ class LineEditWidget(AbstractDataSetWidget):
             if value != old_value:
                 self.edit.setText(value)
         else:
-            self.line_edit_changed(value)
+            # Reset widget if value is None
+            if self.edit.text() != "":
+                self.edit.setText("")
+            else:
+                # Trigger callbacks even if widget is already empty, as the logical
+                # state changed from empty string to None
+                self.line_edit_changed(value)
 
     def line_edit_changed(self, qvalue: str | None) -> None:
         """QLineEdit validator"""
@@ -397,7 +468,7 @@ class LineEditWidget(AbstractDataSetWidget):
         if not self.item.check_value(value):
             self.edit.setStyleSheet("background-color:rgb(255, 175, 90);")
         else:
-            self.edit.setStyleSheet("")
+            self.edit.setStyleSheet(_get_readonly_stylesheet())
             _display_callback(self, value)
         self.update(value)
         self.notify_value_change()
@@ -442,6 +513,7 @@ class TextEditWidget(AbstractDataSetWidget):
         super().__init__(item, parent_layout)
         self.edit = self.group = QTextEdit()
         self.edit.setToolTip(item.get_help())
+        self.edit.setStyleSheet(_get_readonly_stylesheet())
         self.edit.textChanged.connect(self.text_changed)  # type:ignore
 
     def __get_text(self) -> str:
@@ -451,8 +523,7 @@ class TextEditWidget(AbstractDataSetWidget):
     def get(self) -> None:
         """Update widget contents from data item value"""
         value = self.item.get()
-        if value is not None:
-            self.edit.setPlainText(value)
+        self.edit.setPlainText(value or "")  # Reset widget if value is None
         self.text_changed()
 
     def text_changed(self) -> None:
@@ -461,7 +532,7 @@ class TextEditWidget(AbstractDataSetWidget):
         if not self.item.check_value(value):
             self.edit.setStyleSheet("background-color:rgb(255, 175, 90);")
         else:
-            self.edit.setStyleSheet("")
+            self.edit.setStyleSheet(_get_readonly_stylesheet())
         self.update(value)
         _display_callback(self, value)
         self.notify_value_change()
@@ -514,6 +585,9 @@ class CheckBoxWidget(AbstractDataSetWidget):
         value = self.item.get()
         if value is not None:
             self.checkbox.setChecked(value)
+        else:
+            # Reset widget if value is None
+            self.checkbox.setChecked(False)
 
     def set(self) -> None:
         """Update data item value from widget contents"""
@@ -600,6 +674,9 @@ class DateWidget(AbstractDataSetWidget):
             if not isinstance(value, datetime.date):
                 value = datetime.date.fromordinal(value)
             self.dateedit.setDate(value)
+        elif value is None:
+            # Reset widget if value is None
+            self.dateedit.setDate(datetime.date.today())
 
     def set(self) -> None:
         """Update data item value from widget contents"""
@@ -655,6 +732,9 @@ class DateTimeWidget(AbstractDataSetWidget):
             if not isinstance(value, datetime.datetime):
                 value = datetime.datetime.fromtimestamp(value)
             self.dateedit.setDateTime(value)
+        elif value is None:
+            # Reset widget if value is None
+            self.dateedit.setDateTime(datetime.datetime.now())
 
     def set(self) -> None:
         """Update data item value from widget contents"""
@@ -899,9 +979,9 @@ class FileWidget(HLayoutMixin, LineEditWidget):
     ) -> None:
         super().__init__(item, parent_layout)
         self.filedialog = filedialog
-        self.button = QPushButton()
+        self.button = QPushButton(_("Browse..."))
         fmt = item.get_prop_value("data", "formats")
-        self.button.setIcon(get_icon("%s.png" % fmt[0].lower(), default="file.png"))
+        self.button.setIcon(get_icon("%s.png" % fmt[0].lower(), default="FileIcon"))
         self.button.clicked.connect(self.select_file)  # type:ignore
         self.group.addWidget(self.button)
         self.basedir = item.get_prop_value("data", "basedir")
@@ -958,7 +1038,7 @@ class DirectoryWidget(HLayoutMixin, LineEditWidget):
         self, item: "DataItemVariable", parent_layout: "DataSetEditLayout"
     ) -> None:
         super().__init__(item, parent_layout)
-        self.button = QPushButton()
+        self.button = QPushButton(_("Browse..."))
         self.button.setIcon(get_std_icon("DirOpenIcon"))
         self.button.clicked.connect(self.select_directory)  # type:ignore
         self.group.addWidget(self.button)
@@ -1108,6 +1188,12 @@ class ChoiceWidget(AbstractDataSetWidget):
             if self._first_call:
                 self.index_changed(idx)
                 self._first_call = False
+        else:
+            # Reset widget if value is None
+            self.set_widget_value(0)
+            if self._first_call:
+                self.index_changed(0)
+                self._first_call = False
 
     def set(self) -> None:
         """Update data item value from widget contents"""
@@ -1174,6 +1260,9 @@ class MultipleChoiceWidget(AbstractDataSetWidget):
         for (i, _choice, _img), checkbox in zip(_choices, self.boxes):
             if value is not None and i in value:
                 checkbox.setChecked(True)
+            else:
+                # Reset widget if value is None or item not in value
+                checkbox.setChecked(False)
 
     def set(self) -> None:
         """Update data item value from widget contents"""
@@ -1284,10 +1373,16 @@ class FloatArrayWidget(AbstractDataSetWidget):
 
     def get(self) -> None:
         """Update widget contents from data item value"""
-        self.arr = np.asarray(self.item.get())
-        if self.item.get_prop_value("display", "transpose"):
-            self.arr = self.arr.T
-        self.update(self.arr)
+        value = self.item.get()
+        if value is not None:
+            self.arr = np.asarray(value)
+            if self.item.get_prop_value("display", "transpose"):
+                self.arr = self.arr.T
+            self.update(self.arr)
+        else:
+            # Reset widget if value is None
+            self.arr = np.array([])
+            self.update(self.arr)
 
     def update(self, arr: np.ndarray) -> None:
         shape = arr.shape
@@ -1483,7 +1578,7 @@ class DataSetWidget(AbstractDataSetWidget):
         self.set_dataset()
 
     def get_dataset(self) -> None:
-        """update's internal parameter representation
+        """Update's internal parameter representation
         from the item's stored value
 
         default behavior uses update_dataset and assumes
@@ -1493,7 +1588,7 @@ class DataSetWidget(AbstractDataSetWidget):
         update_dataset(self.dataset, item)
 
     def set_dataset(self) -> None:
-        """update the item's value from the internal
+        """Update the item's value from the internal
         data representation
 
         default behavior uses restore_dataset and assumes
@@ -1522,3 +1617,79 @@ class DataSetWidget(AbstractDataSetWidget):
             column_span: number of columns to span
         """
         layout.addWidget(self.group, row, label_column, row_span, column_span + 1)
+
+
+class SeparatorWidget(AbstractDataSetWidget):
+    """SeparatorItem widget
+
+    Displays a horizontal gray line as a visual separator between other items.
+
+    Args:
+        item: instance of `DataItemVariable` (not `DataItem`)
+        parent_layout: parent `DataSetEditLayout` instance
+    """
+
+    def __init__(
+        self, item: DataItemVariable, parent_layout: DataSetEditLayout
+    ) -> None:
+        super().__init__(item, parent_layout)
+
+        # Create a frame to hold the separator line
+        self.group = QFrame()
+        self.group.setFrameShape(QFrame.HLine)
+        self.group.setFrameShadow(QFrame.Sunken)
+
+        # Set the color to gray
+        self.group.setStyleSheet("QFrame { color: gray; }")
+
+        # Set minimum height for visibility
+        self.group.setMinimumHeight(10)
+        self.group.setMaximumHeight(10)
+
+    def get(self) -> None:
+        """Update widget contents from data item value
+
+        Separators don't have values, so this is a no-op.
+        """
+        pass
+
+    def set(self) -> None:
+        """Update data item value from widget contents
+
+        Separators don't have values, so this is a no-op.
+        """
+        pass
+
+    def value(self) -> None:
+        """Returns the widget's current value
+
+        Separators don't have values.
+
+        Returns:
+            None: separators don't have values
+        """
+        return None
+
+    def place_on_grid(
+        self,
+        layout: QGridLayout,
+        row: int,
+        label_column: int,
+        widget_column: int,
+        row_span: int = 1,
+        column_span: int = 1,
+    ) -> None:
+        """Place widget on layout at specified position
+
+        For separators, we span across both label and widget columns.
+
+        Args:
+            layout: parent layout
+            row: row index
+            label_column: column index for label (ignored for separators)
+            widget_column: column index for widget (ignored for separators)
+            row_span: number of rows to span
+            column_span: number of columns to span
+        """
+        # Span across both label and widget columns for full-width separator
+        layout.addWidget(self.group, row, label_column, row_span, column_span + 1)
